@@ -23,7 +23,8 @@ NNS.distance <- function(rpm, rpm_class, dist.estimate, type, k, class){
 
   if(type!="FACTOR"){
     rpm <- rbind(as.list(t(dist.estimate)), rpm[, .SD, .SDcols = 1:n])
-    rpm <- rpm[, names(rpm) := lapply(.SD, function(b) (b - min(b)) / max(1e-10, (max(b) - min(b)))), .SDcols = 1:n]
+    rpm <- rpm[, names(rpm) := lapply(.SD, function(b) ((((b - min(b))^2) / max(1e-10, (max(b) - min(b))^2)) + (((b - min(b))) / max(1e-10, (max(b) - min(b)))))/2),
+               .SDcols = 1:n]
     dist.estimate <- unlist(rpm[1, ])
     rpm <- rpm[-1,]
   }
